//
//  Shader.fsh
//  soyuzSim
//
//  Copyright (c) 2019 Iacopo Baroncini. All rights reserved.
//

precision highp float;
precision lowp sampler2D;

struct DirectionalLight {
    vec3 direction;
    vec4 diffuseColor;
};

struct Material{
    vec4 diffuseFactor;
    float luminance;
};

uniform mat4                u_mMatrix;
uniform mat4                u_mvpMatrix;
uniform Material            u_material;
uniform DirectionalLight    u_light;
uniform DirectionalLight    u_cameraLight;
uniform bool                u_withTexture;
uniform sampler2D           u_sampler;
uniform bool                u_blackAndWhite;
uniform float               u_brightness;

varying vec4                v_position;
varying vec3                v_normal;
varying float               v_diffuseFactor;
varying float               v_diffuseFactor2;
varying vec4                v_diffuseColor;
varying vec2                v_texCoord0;
varying vec3                v_lightDirection;
varying float               v_distance;
varying vec3                v_lightReflected;
varying float               v_reflection;

void main() {
    float ff;
    float ll;
    if (v_distance > 3000.0) {
        // sun
        gl_FragColor = vec4(-1.4, -1.4, -1.2, 0.0) * v_reflection;
    } else {
        gl_FragColor = ((u_withTexture) ? texture2D(u_sampler, v_texCoord0.xy) : u_material.diffuseFactor);
        if (v_distance < 200.0) {
            // iss, soyuz, progress
            gl_FragColor *= (u_light.diffuseColor * v_diffuseFactor) + (u_cameraLight.diffuseColor * v_diffuseFactor2);
        } else
        if (v_distance < 3000.0) {
            // earth
            float water = dot(vec4(-4.3, 2.0, 2.0, 0.0), gl_FragColor) * 0.60;
            if (v_distance > 1600.0) {
                ll = length(u_light.diffuseColor);
                gl_FragColor += ((v_distance - 1600.0) / 700.0) * (vec4(0.0, 0.5, 1.2, 0.0) * ll + vec4(0.1, 0.3, 0.1, 0.0) * (1.0 - ll));
            }
            ff = dot(normalize(vec3(v_position)), v_lightReflected);
            if (ff > 0.75 && water > 0.0)
                gl_FragColor += vec4(1.0, 0.6, 0.4, 0.0) * (water * pow(ff, 20.0));
        }
        gl_FragColor *= u_brightness;
        if (!u_blackAndWhite) return;
        float luminance = dot(gl_FragColor, vec4(0.299, 0.587, 0.114, 0.0));
        gl_FragColor = vec4(luminance, luminance, luminance, 1.0);
    }
}

