//
//  Shader.vsh
//  soyuzSim
//
//  Copyright (c) 2019 Iacopo Baroncini. All rights reserved.
//

precision highp float;
precision lowp sampler2D;

struct DirectionalLight {
    vec3 direction;
    vec4 diffuseColor;
};

struct Material{
    vec4 diffuseFactor;
    float luminance;
};

attribute vec4              a_position;
attribute vec3              a_normal;
attribute vec2              a_texCoord0;

uniform mat4                u_mMatrix;
uniform mat4                u_mvpMatrix;
uniform Material            u_material;
uniform DirectionalLight    u_light;
uniform DirectionalLight    u_cameraLight;
uniform float               u_brightness;

varying float               v_diffuseFactor;
varying float               v_diffuseFactor2;
varying vec2                v_texCoord0;
varying vec4                v_position;
varying vec3                v_normal;
varying float               v_distance;
varying vec3                v_lightReflected;
varying float               v_reflection;

void main() {
    v_normal = normalize(vec3(u_mMatrix * vec4(a_normal, 0.0)));
    v_position = u_mMatrix * a_position;
    v_distance = length(v_position);
    v_reflection = dot(u_light.direction, v_normal);
    v_diffuseFactor = max(u_material.luminance, v_reflection); // luminance
    v_diffuseFactor2 = max(u_material.luminance, dot(u_cameraLight.direction, v_normal));
    v_texCoord0 = a_texCoord0;
    v_lightReflected = u_light.direction + 2.0 * v_normal * dot(-v_normal, u_light.direction);
    gl_Position = u_mvpMatrix * a_position;
}